<?php

/**
 * Query printer using D3.js library for creating a portfolio for technology assessment.
 */


class SRFPortfolio extends SMWAggregatablePrinter {

	/**
	 * Get a human readable label for this printer.
	 * @see SMWResultPrinter::getName()
	 * 
	 * @return string
	 */
	public function getName() {
		// Add your result printer name here
		return wfMessage( 'srf-printername-portfolio' )->text();
	}

	/**
	 * @see SMWResultPrinter::getResultText
	 *
	 * @param SMWQueryResult $result
	 * @param $outputMode
	 *
	 * @return string
	 */
	protected function getResultText( SMWQueryResult $result, $outputMode ) {
		// Data processing
				$data = array();

		while ( $row = $result->getNext() ) {
			// Loop over their fields (properties)
			foreach ( $row as /* SMWResultArray */ $field ) {

				$rowNumbers = array();

				// Property label
				$groupedBy = $field->getResultSubject()->getTitle()->getText(); 

				// Loop over all values for the property.
				while ( ( /* SMWDataValue */ $object = $field->getNextDataValue() ) !== false ) {

					if ( $object->getDataItem()->getDIType() == SMWDataItem::TYPE_NUMBER ) {
						$number =  $object->getNumber();

						if ( $label !== '' ) {

							// Reference array summarize all items per row
							$rowNumbers += array ( 'subject' => $label, 'value' => $number );
							$data[$groupedBy][] = $number;

						}
					}
					
					else {
						$label = $object->getWikiValue();
					}
				}
			}
		}

		// Check if the data processing returned any results otherwise just bailout
		if ( $data === array() ) {
			// Add an error message to return method
			return $result->addErrors( array( wfMessage( 'srf-no-results' )->inContentLanguage()->text() ) );
		
		} else {

			$options = array(
				'mode' => $outputMode
			);
			// Return formatted results
			return $this->getFormatOutput( $data );
		}
	}


	/**
	 * Prepare data for the output
	 *
	 * @since 1.8
	 *
	 * @param array $data
	 * @param array $options
	 *
	 * @return string
	 */
	protected function getFormatOutput( array $data ) {
		
		// The generated ID is to distinguish similar instances of the same
		// printer that can appear within the same page
		static $statNr = 0;
		$portfolioID = 'srf-portfolio-' . ++$statNr;


		// Used to set that the output and being treated as HTML (opposed to plain wiki text)
		$this->isHTML = true;
		// Reorganize the raw data
		foreach ( $data as $name => $value ) {
			if ( true ) {
				$dataObject[] = array( 'label' => $name , 'value' => $value );
			}
		}

		// Ensure right conversion
		$width = strstr( $this->params['width'] ,"%") ? $this->params['width'] : $this->params['width'] . 'px';

		// Prepare transfer objects
		$portfoliodata = array (
			'data' => $dataObject,
			'parameters' => array (
			'caliber'   => $this->params['caliber']
			)

		);

		// Correct escaping is vital to minimize possibilites of malicious code snippets
		// and also a coherent string evalution therefore it is recommended
		// that data transferred to the JS plugin is JSON encoded
		// Assign the ID to make a data instance readly available and distinguishable
		// from other content within the same page
		$requireHeadItem = array ( $portfolioID => FormatJson::encode( $portfoliodata ) );
		SMWOutputs::requireHeadItem( $portfolioID, Skin::makeVariablesScript( $requireHeadItem ) );

		// Add resource definitions that has been registered with SRF_Resource.php
		// Resource definitions contain scripts, styles, messages etc.
		// SMWOutputs::requireResource( 'ext.srf.portfolio.namespace' );
		SMWOutputs::requireResource( 'ext.srf.portfolio.namespace' );

		// Prepares an HTML element showing a rotating spinner indicating that something
		// will appear at this placeholder. The element will be visible as for as
		// long as jquery is not loaded and the JS plugin did not hide/removed the element.
		$processing = SRFUtils::htmlProcessingElement();

		// Add two elements a outer wrapper that is assigned a class which the JS plugin
		// can select and will fetch all instances of the same result printer and an innner
		// container which is set invisible (display=none) for as long as the JS plugin
		// holds the content hidden. It is normally the place where the "hard work"
		// is done hidden from the user until it is ready.
		// The JS plugin can prepare the output within this container without presenting
		// unfinished visual content, to avoid screen clutter and improve user experience.
		// Chart/graph placeholder
		$chart = Html::rawElement( 'div', array(
			'id'    => $portfolioID,
			'class' => 'container',
			), null
		);

		// Processing placeholder
		$processing = SRFUtils::htmlProcessingElement( $this->isHTML );

		// D3 wrappper
		return Html::rawElement( 'div', array(
			'class' => 'srf-portfolio',
			'style' => "width:{$width}; height:{$this->params['height']}px;"
			), $processing . $chart
		);
	}

	/**
	 * @see SMWResultPrinter::getParamDefinitions
	 *
	 * @since 1.8
	 *
	 * @param $definitions array of IParamDefinition
	 *
	 * @return array of IParamDefinition|array
	 */
	public function getParamDefinitions( array $definitions ) {
		$params = parent::getParamDefinitions( $definitions );

		$params['min'] = array(
			'type' => 'integer',
			'message' => 'srf-paramdesc-minvalue',
			'default' => false,
			'manipulatedefault' => false,
		);

		$params['height'] = array(
			'type' => 'integer',
			'message' => 'srf_paramdesc_chartheight',
			'default' => 400,
			'lowerbound' => 1,
		);

		$params['width'] = array(
			'message' => 'srf_paramdesc_chartwidth',
			'default' => 400,
		);

		$params['caliber'] = array(
			'message' => 'srf_paramdesc_caliber',
			'default' => '',
		);

		return $params;
	}
}



